from pathlib import Path
from typing import Tuple
import pandas as pd
from geoprob_pipe.utils.file_system import FileSystem
from datetime import datetime
# noinspection PyPep8Naming
from geoprob_pipe.utils.loggers import TmpAppConsoleHandler as logger


class Workspace:
    """Workspace class which handles all actions related to input, output and intermediate working files"""

    def __init__(self, path_workspace: str | Path) -> None:
        """Initialize Workspace instance

        Args:
            path_workspace (str | Path): path to the folder that contains all required input and where all output and
                working files will be stored
        """
        self.folderpath = FileSystem.validate_path(path_workspace)
        self.path_output_folder = _prepare_output_folder(self.folderpath)
        self.input, self.path_input_excel, self.path_hrd = _prepare_input_folder(self.folderpath)
        # TODO Later Could Groot: Add functionality to read existing results (without running prob. calculations again)
        logger.info("Workspace (I/O folders) successfully processed.")

    def update_output_filesystem(self) -> None:
        """
        Update the output subfolder FileSystem instance (to include new calculation results)
        """
        self.path_output_folder = _prepare_output_folder(self.folderpath)


def _prepare_output_folder(path_workspace: Path) -> FileSystem:
    """ Prepare output subfolder

    Args:
        path_workspace (Path): path to the folder that contains all required input and where all output and working
            files will be stored

    Raises:
        FileNotFoundError: raised if USE_EXISTING_TKX_RESULTS=True but the output subfolder contains no .tkx files
        FileExistsError: raised if USE_EXISTING_TKX_RESULTS=False but the output subfolder still contains .tkx files
            from a previous run

    Returns:
        FileSystem: _description_
    """
    timestamp = datetime.now().strftime("%Y-%m-%d_%H%M")
    export_path = path_workspace / "output" / timestamp
    export_path.mkdir(exist_ok=False, parents=True)
    logger.info(f"Output folder was successfully created in workspace folder: output/{timestamp}/.")
    return FileSystem(export_path)


def _prepare_input_folder(path_workspace: Path) -> Tuple[FileSystem, Path, Path]:

    """Prepare input subfolder

    Args:
        path_workspace (Path): path to the folder that contains all required input and where all output and working
            files will be stored

    Raises:
        FileNotFoundError: raised if USE_EXISTING_TKX_RESULTS=True but the number of .stix files in the input subfolder
            don't match the number of .tkx files in the output subfolder
        FileNotFoundError: raised if USE_EXISTING_TKX_RESULTS=False but the input subfolder contains no .stix files
        FileNotFoundError: raised if USE_EXISTING_TKX_RESULTS=False but the input subfolder contains no template .tkx
            file

    Returns:
        FileSystem: FileSystem instance which creates a convenient overview of the input subfolder and the files within it
    """
    # Create input folder if it didn't exist yet
    if not Path(path_workspace / "input").exists():
        Path.mkdir(path_workspace / "input", parents=False, exist_ok=False)
        print(f"INFO: input folder was successfully created in project folder ({path_workspace / 'input'})")

    # Create FileSystem instance for input folder
    filesystem_input = FileSystem(path_workspace / "input")

    # Make sure the "input" subfolder contains 1 .xlsx file named input.xlsx
    xlsx_input = [filepath.name
                  for filepath in FileSystem.find_files_in_dir(filesystem_input.folderpath, "xlsx")]

    if len(xlsx_input) != 1 or xlsx_input[0] != "input.xlsx":
        raise FileNotFoundError(
            f"\nInput folder {filesystem_input.folderpath} should contain exactly 1 .xlsx files named 'input.xlsx'\n"
            f"(Currently found: {xlsx_input})"
        )

    # Check if input.xlsx has exactly four required sheets (case-sensitive!)
    expected_sheets = {"Vakken", "Uittredepunten", "Ondergrondscenarios", "Overzicht_parameters", "Settings"}
    xlsx_input_folderpath = pd.ExcelFile(filesystem_input.folderpath / "input.xlsx")

    if set(xlsx_input_folderpath.sheet_names) != expected_sheets:
        raise FileNotFoundError(
            f"\n{filesystem_input.folderpath / 'input.xlsx'} should contain exactly these sheets:\n"
            f"{', '.join(expected_sheets)}\n"
            f"Found sheets: {', '.join(xlsx_input_folderpath.sheet_names)}\n"
            f"Note:\n1) Sheet names are case-sensitive!\n2) The order of sheets does not matter."
        )
        
    # Find HRD path
    hrd_matches = filesystem_input.files.loc[
        lambda df: df["filename"].str.endswith(".sqlite") &
                (df["filename"] != "hlcd.sqlite") &
                ~df["filename"].str.endswith(".config.sqlite")
    ]
    # The .zip files containing HRD have multiple .sqlite files, but we only want the one that ends with .sqlite (not
    # ending with .config.sqlite) and is not called 'hlcd.sqlite'

    if len(hrd_matches) != 1:
        if len(hrd_matches) > 1:
            filepaths = ', '.join(hrd_matches["filepath"].tolist())
            raise ValueError(f"Expected exactly one matching HRD .sqlite file, but found {len(hrd_matches)}:\n{filepaths}")
        else:
            raise ValueError("One HRD .sqlite file is required in the input folder, but none was found.")

    hrd_path = hrd_matches["filepath"].iloc[0]
        
    return filesystem_input, filesystem_input.folderpath / "input.xlsx", hrd_path



from __future__ import annotations
import os
import numpy as np
import pandas as pd
import plotly.colors as pc
from plotly.graph_objects import Figure, Scatter
from typing import TYPE_CHECKING

from geoprob_pipe.cmd_app.parameter_input.expand_input_tables import run_expand_input_tables

if TYPE_CHECKING:
    from geoprob_pipe import GeoProbPipe


def river_waterlevel(geoprob_pipe: GeoProbPipe, export: bool = False):
    # Prepare base data
    df = geoprob_pipe.results.df_alphas_influence_factors_and_physical_values(
        system_only=True, filter_deterministic=False, filter_derived=False
    )

    df = df[[
        "uittredepunt_id", "ondergrondscenario_id", "vak_id",
        "variable", "distribution_type", "physical_value"
    ]]

    gdf_uittredepunten = geoprob_pipe.input_data.uittredepunten.gdf
    df = df.merge(
        gdf_uittredepunten[["uittredepunt_id", "metrering", "hrd_name"]],
        on="uittredepunt_id", how="left"
    )
    df_beta = geoprob_pipe.results.df_beta_uittredepunten

    # Target exceedance frequencies
    target_freqs = np.array([
        0.1, 0.033333333, 0.01, 0.003333333,
        0.001, 0.000333333, 0.0001, 3.33333E-05,
        0.00001, 3.33333E-06
    ])

    # Blue gradient for lines
    line_colors = pc.sample_colorscale(
        "Jet", np.linspace(0.2, 0.9, len(target_freqs))
        )
    freq_color_map = {f: c for f, c in zip(target_freqs, line_colors)}

    # Figure
    fig = Figure()

    # Prepare storage for Hydra curves per frequency
    hydra_curves = {freq: {"metrering": [], "level": []}
                    for freq in target_freqs}

    # Add Hydra lines (grouped per frequency)
    df_input: pd.DataFrame = run_expand_input_tables(
        geoprob_pipe.input_data.app_settings.geopackage_filepath,
        add_frag_ref=True)
    df_input = df_input[df_input["parameter_name"] == "buitenwaterstand"]
    df_input = pd.concat(
        [df_input.drop(columns=['parameter_input']),
         df_input['parameter_input'].apply(pd.Series)], axis=1)

    for _, row in df_input.iterrows():
        if row["distribution_type"] != "deterministic":
            df_subset = gdf_uittredepunten[
                gdf_uittredepunten["vak_id"] == row["vak_id"]
                ]
            if df_subset.empty:
                continue

            m_values = df_subset["metrering"].to_numpy()

            # Hydra exceedance curve
            freqs = np.array(
                [fv.probability_of_failure for fv in row.loc["fragility_values"]],
                dtype=float
                )
            levels = np.array(
                [fv.x for fv in row.loc["fragility_values"]],
                dtype=float
                )

            # Sort for interpolation
            sort_idx = np.argsort(freqs)
            freqs = freqs[sort_idx]
            levels = levels[sort_idx]

            # Interpolate levels for standard frequencies
            interp_levels = np.interp(target_freqs, freqs, levels)

            # Store values for each frequency
            for freq, level in zip(target_freqs, interp_levels):
                hydra_curves[freq]["metrering"].extend(m_values)
                hydra_curves[freq]["level"].extend(np.full_like(m_values, level))

    # Plot one continuous line per exceedance frequency

    for freq, data in hydra_curves.items():
        # Sort by metrering for continuous line plotting
        sort_idx = np.argsort(data["metrering"])
        m_sorted = np.array(data["metrering"])[sort_idx]
        level_sorted = np.array(data["level"])[sort_idx]

        fig.add_trace(
            Scatter(
                x=m_sorted,
                y=level_sorted,
                mode="lines",
                line=dict(color=freq_color_map[freq], width=2),
                name=f"1/{1/freq:,.0f}".replace(",", "."),  # mark thousands
                showlegend=True,
            )
        )

    # Buitenwaterstand markers with β color scale
    df_filtered = df[df["variable"] == "buitenwaterstand"].merge(
        df_beta[["uittredepunt_id", "beta"]],
        on="uittredepunt_id", how="left"
    )

    betas = df_filtered["beta"].to_numpy()

    # Green-to-red scale (low beta = red, high beta = green)
    beta_colorscale = "RdYlGn"
    fig.add_trace(
        Scatter(
            x=df_filtered["metrering"],
            y=df_filtered["physical_value"],
            mode="markers",
            name="Physical values",
            marker=dict(
                symbol="circle",
                size=7,
                color=betas,
                colorscale=beta_colorscale,
                cmin=1,
                cmax=10,
                colorbar=dict(
                    title="β",
                    xanchor="right",
                    x=-0.05,  # Make space on the left side for y-axis
                    ticks="outside",
                ),
                line=dict(width=0.5, color="black"),
            ),
            customdata=df_filtered[["beta"]],
            hovertemplate=("metering: %{x}<br>" +
                           "buitenwaterstand: %{y:.2f}<br>" +
                           "beta: %{customdata:.3f}"),
            showlegend=True
        )
    )

    # Layout
    fig.update_layout(
        title="Buitenwaterstanden bij herhaaltijd en physical values uit system design points",
        xaxis=dict(title="Metrering [m]", showgrid=True,
                   gridwidth=0.5, gridcolor="gray"),
        yaxis=dict(title="Buitenwaterstand [m+NAP]", showgrid=True,
                   gridwidth=0.5, gridcolor="gray"),
        legend=dict(
            orientation="v",
            yanchor="middle",
            y=0.5,
            xanchor="left",
            x=1.02,
            title="Legenda",
            bgcolor="rgba(255,255,255,0.7)",
        ),
        legend_traceorder="reversed",
        height=600,
        margin=dict(r=200),
    )

    # Export
    if export:
        export_dir = os.path.join(
            geoprob_pipe.visualizations.graphs.export_dir,
            "grafiek_physical_values"
        )
        os.makedirs(export_dir, exist_ok=True)

        fig.write_html(os.path.join(export_dir, "river_waterlevel.html"),
                       include_plotlyjs='cdn')

    return fig



from __future__ import annotations
from geoprob_pipe.utils.statistics import convert_failure_probability_to_beta
import pandas as pd
from typing import TYPE_CHECKING, List
if TYPE_CHECKING:
    from geoprob_pipe.results import Results
    from geoprob_pipe import GeoProbPipe
    from geoprob_pipe.calculations.systems.base_objects.system_calculation import \
        SystemCalculation
    from probabilistic_library import DesignPoint
    from geoprob_pipe.calculations.systems.build_and_run import CalcResult


def collect_df_beta_limit_state(calculation: SystemCalculation) -> pd.DataFrame:

    def create_row(calc, dp: DesignPoint, model_name):
        return {
            "uittredepunt_id": calc.metadata["uittredepunt_id"],
            "ondergrondscenario_id": calc.metadata["ondergrondscenario_naam"],  # TODO: id naar naam veranderen?
            "vak_id": calc.metadata["vak_id"],
            "limit_state": model_name,
            "converged": dp.is_converged,
            "beta": round(dp.reliability_index, 2),
            "failure_probability": dp.probability_failure,
            "convergence": dp.convergence,
            "total_iterations": dp.total_iterations,
            "total_model_runs": dp.total_model_runs,
        }

    rows = []
    for design_point, model in zip(calculation.model_design_points, calculation.given_limit_states):
        rows.append(create_row(calc=calculation, dp=design_point, model_name=model.__name__))
    df = pd.DataFrame(rows).sort_values(by=["uittredepunt_id", "ondergrondscenario_id", "vak_id"]).reset_index(drop=True)
    return df


def combine_df_beta_per_limit_state(calc_results: List[CalcResult]) -> pd.DataFrame:
    df = pd.concat((result.df_limit_state for result in calc_results), ignore_index=True)
    return df


def collect_df_beta_scenario(calc: SystemCalculation) -> pd.DataFrame:

    def create_row(calculation):
        return {
            "uittredepunt_id": calculation.metadata["uittredepunt_id"],
            "ondergrondscenario_id": calculation.metadata["ondergrondscenario_naam"],  # TODO: id naar naam veranderen?
            "vak_id": calculation.metadata["vak_id"],
            "system_calculation": calculation,
            "converged": calculation.system_design_point.is_converged,
            "beta": round(calculation.system_design_point.reliability_index, 2),
            "failure_probability": calculation.system_design_point.probability_failure,
            "convergence": calculation.system_design_point.convergence,
            "total_model_runs": calculation.system_design_point.total_model_runs,
            "total_iterations": calculation.system_design_point.total_iterations,
            "model_betas": ", ".join([
                str(round(dp.reliability_index, 2)) for dp in calculation.model_design_points
            ])
        }
    row = create_row(calc)

    return pd.DataFrame([row])


def combine_df_beta_per_scenario(calc_results: List[CalcResult]) -> pd.DataFrame:
    df = pd.concat((result.df_scenario for result in calc_results), ignore_index=True)
    df = df.sort_values(["uittredepunt_id", "ondergrondscenario_id", "vak_id"]).reset_index(drop=True)
    return df


def calculate_df_beta_per_uittredepunt(geoprob_pipe: GeoProbPipe, results: Results) -> pd.DataFrame:

    # Sum
    df = results.df_beta_scenarios.assign(
        failure_probability=results.df_beta_scenarios.apply(
            lambda row: row['failure_probability'] * geoprob_pipe.input_data.scenarios.scenario_kans(
                vak_id=row['vak_id'], scenario_naam=row['ondergrondscenario_id']
            ), axis=1)).groupby('uittredepunt_id', as_index=False)[
        'failure_probability'].sum()
    df["beta"] = df["failure_probability"].apply(lambda failure_prob: convert_failure_probability_to_beta(failure_prob))

    # Determine when uittredepunt is converged (when all scenarios are converged)
    conv = results.df_beta_scenarios.groupby(
        'uittredepunt_id', as_index=False)["converged"].all()
    df = df.merge(conv, on="uittredepunt_id", how="left")

    # Add vak id back to it
    gdf_uittredepunten = geoprob_pipe.input_data.uittredepunten.gdf
    df_uittredepunten = gdf_uittredepunten[["uittredepunt_id", "vak_id"]]
    df = df.merge(df_uittredepunten, left_on="uittredepunt_id", right_on="uittredepunt_id")

    return df[["uittredepunt_id", "vak_id", "converged", "beta", "failure_probability"]]


def construct_df_beta_per_vak(results: Results):

    # TODO: Check if all calculations on scenario level are converged?
    conv = results.df_beta_scenarios.groupby('vak_id', as_index=False)["converged"].all()

    # TODO: Wat doet dit stukje code?
    df = results.df_beta_uittredepunten
    df = df.drop(columns=["converged"])  # TODO: Waarom drop converged?
    df = df.loc[df.groupby('vak_id')['beta'].idxmin()]  # Minimale beta van beta uittredepunten per vak

    # TODO: Waarom de merge?
    df = df.merge(conv, on="vak_id", how="left")
    return df[["uittredepunt_id", "vak_id", "converged", "beta", "failure_probability"]]




CHANGELOG = {
    "1.3.1":
        "Hotfix. Visualisatie en export issues. Geen issue aan berekeningen "
        "zelf. 1e issue: Foute beta waarden in de hover van de "
        "betrouwbaarheidsindex.html. 2e issue: De df_limit_states werd "
        "geëxporteerd als df_scenarios.",
    "1.3.0":
        "Probabilistic Library nu toegevoegd als 'setup dependency' nadat "
        "Deltares deze heeft vrijgegeven op PyPI. Eveneens Chrome "
        "uitgefaseerd als software requirement. Daardoor zijn beiden geen "
        "handmatige software requirements meer.",
    "1.2.0":
        "Implementatie van parallel rekenen en het inspecteren van een enkele "
        "berekening.",
    "1.1.0":
        "Implementatie van een eerste versie van het vergelijken van twee "
        "verschillende GeoProb-Pipe bestanden.",
    "1.0.0":
        "Implementatie GeoPackage als data (in- en uitvoer) bestand én "
        "implementatie van keuze in geohydrologische modellen.",
    "0.1.0":
        "Toevoegen features na interne gebruikerssessie WSRL 2025-08-11: (a) "
        "geopackage-export met resultaten, (b) Excel-export met physical "
        "values, alphas en invloedsfactoren, (c) Excel-export met resultaat "
        "per vak, (d) Excel-export met validatie berichten en (e) physical "
        "values in grafiek waterstandssfrequentielijn.",
    "0.0.15":
        "Eerste vrijgave van wheel installatie-bestand.",
}



from __future__ import annotations
from typing import TYPE_CHECKING, List
from geoprob_pipe.calculations.systems.mappers.calculation_mapper import (
    CALCULATION_MAPPER)
from multiprocessing import Pool, cpu_count
import time
import math
from dataclasses import dataclass
from geoprob_pipe.results.construct_dataframes import (
    collect_df_beta_limit_state, collect_df_beta_scenario)
from geoprob_pipe.results.alphas_and_physical_values import (
    collect_stochast_values, calculate_derived_values)
# noinspection PyPep8Naming
from geoprob_pipe.utils.loggers import TmpAppConsoleHandler as logger

if TYPE_CHECKING:
    from pandas import DataFrame
    from geoprob_pipe import GeoProbPipe
    from geoprob_pipe.calculations.systems.base_objects\
        .base_system_build import BaseSystemBuilder
    from geoprob_pipe.utils.validation_messages import ValidationMessages

_BUILDER: BaseSystemBuilder
_MODEL: str


@dataclass
class CalcResult:
    """
    Dataclass om de resultaten te verzamelen vanuit de calculation.
    Bevat de volgende attributen:
    Dataframe: df_limit_state,
    Dataframe: df_scenario,
    Dataframe: df_stochast,
    Dataframe: df_derived,
    ValidationMessages: validation_message
    """
    df_limit_state: DataFrame
    df_scenario: DataFrame
    df_stochast: DataFrame
    df_derived: DataFrame
    validation_message: ValidationMessages


def _init_worker(geohydrologisch_model, geopackage_filepath,
                 to_run_vakken_ids):
    """ Initiator voor de worker, dit zorgt ervoor dat de tijdrovende
    stappen een keer per worker worden uitgevoerd en dan beschikbaar blijven
    voor iedere run.
    """
    global _BUILDER, _MODEL
    _MODEL = geohydrologisch_model
    _BUILDER = (
        CALCULATION_MAPPER[geohydrologisch_model]["system_builder"](
            geopackage_filepath=geopackage_filepath,
            to_run_vakken_ids=to_run_vakken_ids))


def _worker(row_unique: dict):
    """ De worker functie die op de parallelle rekenkernen wordt gedraaid.
    """

    # Build and run calculations
    calc = _BUILDER.build_instance(row_unique=row_unique)
    calc.run()

    # Collect results
    df_limit_state = collect_df_beta_limit_state(calc)
    df_scenario = collect_df_beta_scenario(calc)
    df_stochast = collect_stochast_values(calc)
    df_derived = calculate_derived_values(df_scenario, _MODEL)
    df_scenario = df_scenario.drop(columns=["system_calculation"])

    # Return results (without calculation object)
    return CalcResult(df_limit_state, df_scenario, df_stochast, df_derived,
                      calc.validation_messages)


def build_and_run_system_calculations(
    geoprob_pipe: GeoProbPipe
        ) -> List[CalcResult]:
    """ In deze functie worden de parameters voor de berekeningen verzamelt,
    aan de workers gegeven en vervolgens de resultaten verzameld.
    """
    geohydrologisch_model = geoprob_pipe.input_data.geohydrologisch_model
    geopackage_filepath = (
        geoprob_pipe.input_data.app_settings.geopackage_filepath)
    to_run_vakken_ids = geoprob_pipe.input_data.app_settings.to_run_vakken_ids
    system_builder: BaseSystemBuilder = (
        CALCULATION_MAPPER[geohydrologisch_model]['system_builder'](
            geopackage_filepath=geopackage_filepath,
            to_run_vakken_ids=to_run_vakken_ids))

    logger.info("Now building and running calculations...")
    df_unique_combos = system_builder.setup_iteration_df()
    # Bepaal de parameters voor de multiprocessing setup en de logger
    n_threads: int = cpu_count() - 1
    n_calc_totaal: int = len(df_unique_combos)
    # Minimaal 5 berekeningen per chunk en grootte van chunk beperken
    # zodat er gelogd kan worden.
    chunk_size: int = max(math.ceil(n_calc_totaal / (n_threads * 10)), 5)
    logger.info(
        f"Running {n_calc_totaal} calculations in chunks of {chunk_size}"
        f" with {n_threads} parallel threads.")
    char_len_total = str(n_calc_totaal).__len__()
    logger.info(
        f"Progress: {0:>{char_len_total}} / {n_calc_totaal} calculations.")

    # Dicts zijn gemakkelijker te pickelen en daardoor sneller te
    # verwerken dan pandas series.
    rows = [dict(zip(df_unique_combos.columns, r))
            for r in df_unique_combos.itertuples(index=False, name=None)]

    last_report = time.time()
    done = 0
    results: List[CalcResult] = []
    pool_size = max(min(math.floor(n_calc_totaal / chunk_size), n_threads), 1)

    # Multiprocessing setup
    with Pool(processes=pool_size, initializer=_init_worker, initargs=(
            geohydrologisch_model, geopackage_filepath, to_run_vakken_ids
            )) as pool:

        for res in pool.imap_unordered(_worker, rows, chunksize=chunk_size):
            results.append(res)
            done += 1

            # Alleen kijken of er gelogd moet worden bij de laatste
            # berekening die uit de chunk komt.
            if done % chunk_size != 0:
                continue

            # Alleen loggen wanneer 30 seconden is gepasseerd
            now = time.time()
            if now - last_report < 30.0:
                continue

            # Log
            logger.info(f"Progress: {done:>{char_len_total}} / {n_calc_totaal}"
                        " calculations.")
            last_report = now

    return results



import typer
from rich.console import Console
import os
from geoprob_pipe.utils import clear_terminal
from datetime import datetime
from rich.panel import Panel
from geoprob_pipe.cmd_app.questionnaire import start_questionnaire
from typing import Optional, List
from geoprob_pipe.cmd_app.utils.misc import get_geoprob_pipe_version_number


app = typer.Typer(help="GeoProb-Pipe - CLI applicatie voor probabilistische piping berekeningen.")


class ApplicationSettings:

    def __init__(self):
        self.workspace_dir: Optional[str] = None
        self.geopackage_filename: Optional[str] = None
        self.datetime_stamp: str = datetime.now().strftime("%Y-%m-%d_%H%M%S")
        self.to_run = "all"
        # -> or vakken:1,2,3,4,5

    @property
    def geopackage_filepath(self) -> str:
        return os.path.join(self.workspace_dir, self.geopackage_filename)

    @property
    def hrd_dir(self):
        path_to_hrd_dir = os.path.join(self.workspace_dir, "hrd_files")
        os.makedirs(path_to_hrd_dir, exist_ok=True)
        return path_to_hrd_dir

    @property
    def hrd_file_path(self) -> str:
        for file in os.listdir(self.hrd_dir):
            filename = os.fsdecode(file)
            if filename.endswith(".config.sqlite"):
                continue
            if filename.endswith("hlcd.sqlite"):
                continue
            return os.path.join(self.hrd_dir, filename)
        raise ValueError

    @property
    def ahn_filepath(self) -> str:
        return os.path.join(self.workspace_dir, "ahn", "ahn.tif")

    @property
    def to_run_vakken_ids(self) -> Optional[List[int]]:
        if self.to_run == "all":
            return None
        vak_ids_str: List[str] = self.to_run.replace("vakken:", "").split(sep=",")
        return [int(vak_id_str) for vak_id_str in vak_ids_str]


@app.command()
def startup_geoprob_pipe():
    """ Starts up the GeoProb-Pipe console application. """
    clear_terminal()

    console = Console()
    console.print(Panel(
        """
Welkom bij GeoProb-Pipe! Deze applicatie voert probabilistische pipingberekeningen uit met de uittredepuntenmethode en 
een geohydrologisch model naar keuze (zoals model4a). GeoProb-Pipe maakt gebruik van de probabilistische bibliotheek 
van Deltares, die onder de motorkap de PTK-tool aanstuurt. Met de onderstaande interactieve vragenmodule neemt 
GeoProb-Pipe je stap voor stap mee door het opzetten van de invoer en het uitvoeren van de berekeningen. 
""",
        title=f"GeoProb-Pipe ({get_geoprob_pipe_version_number()})".upper(),
        title_align="left",
        border_style="bright_blue",
        padding=(0, 2),
    ))

    start_questionnaire(ApplicationSettings())


if __name__ == "__main__":
    startup_geoprob_pipe()



# import logging
# from colorlog import ColoredFormatter
from datetime import datetime
from geoprob_pipe.utils.validation_messages import BColors

# def initiate_app_logger(to_console: bool = True):
#
#     # Initiate logger
#     logger = logging.getLogger("geoprob_pipe_logger")
#     logger.setLevel(logging.INFO)
#
#     # Console handler
#     if to_console:
#         console_handler = logging.StreamHandler()
#         console_handler.setLevel(logging.INFO)
#         formatter = ColoredFormatter(
#             "%(log_color)s%(asctime)s - %(levelname)s - %(message)s",
#             datefmt= "%Y-%m-%d %H:%M:%S",
#             log_colors={
#                 "INFO": "green",
#             }
#         )
#         console_handler.setFormatter(formatter)
#         logger.addHandler(console_handler)


class TmpAppConsoleHandler:

    @staticmethod
    def info( msg: str):
        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        print(BColors.OKGREEN, f"{timestamp} - INFO - {msg}", BColors.ENDC)
